# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-29 13:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=50)),
                ('some_number', models.IntegerField()),
                ('some_date', models.DateField()),
            ],
        ),
    ]
